/*
 * Hexecute 3.0
 * FileOpen.cpp
 * copyright 2000 Mike Walston
 *
 * This is the code that opens a new file for Hexecute
 */
#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <tchar.h>
#include "resource.h"
#include "hexecute_defines.h"
#include "hexecute_globals.h"
#include "hexecute_types.h"

int OpenFile(HWND hWnd)
{
	MENUITEMINFO mii;
	mii.cbSize = sizeof(MENUITEMINFO);
	mii.fType = MFT_STRING;          
	mii.fState = MFS_ENABLED;          
	mii.fMask = MIIM_TYPE | MIIM_ID | MIIM_STATE;

// structure needed for calling windows' open file dialog
	int p00pyc0unter = 0;
	OPENFILENAME ofn;
	LPTSTR path = (LPTSTR)malloc(_MAX_PATH); 
	_tcscpy((TCHAR*)path, TEXT(""));
	char *Caption, szDrive[_MAX_PATH], szDir[_MAX_PATH], 
			szName[_MAX_PATH], szExt[_MAX_PATH], szPath[_MAX_PATH];
	char temp[100];
//	HMENU MenuTemp;

	Caption = new char[MAX_PATH];
	// zero out ofn and fill it with thte wanted values
	ZeroMemory(path, _MAX_PATH);
	ZeroMemory(&ofn, sizeof(OPENFILENAME));
    ofn.lStructSize			= sizeof(OPENFILENAME);
	ofn.hwndOwner 			= hWnd;
	ofn.lpstrFilter			= Filters;
	ofn.nMaxFile 			= MAX_PATH;
	ofn.nMaxFileTitle		= MAX_PATH;
	ofn.lpstrFile			= path;
	ofn.lpstrFileTitle 		= TitleName;
	ofn.lpstrTitle	  		= "Open a file";
	ofn.Flags				= OFN_EXPLORER | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;

	if(!GetOpenFileName(&ofn))
	{
		// they cancelled, so let's just leave
		free(path);
		delete[] Caption;
		return FILE_CANCELLED;
	}
					
	// let's put the file name in the window title caption
	wsprintf(Caption, "%s %d.%d.%d - [ ", PROGRAM_NAME, PROGRAM_MAJOR, PROGRAM_MINOR, PROGRAM_BUILD);
	strcat(Caption, ofn.lpstrFileTitle);
	strcat(Caption, " ]");
	_tcscpy(Filename, ofn.lpstrFile);
	_tcscpy(TitleName, ofn.lpstrFileTitle);
	strcat(Caption, " - ");
	if(CleanUp)
		strcat(Caption, "ASCII");
	else
		strcat(Caption, CPages[CurrentCodePage].name); 
	FileStream.open(ofn.lpstrFile, std::ios::in | std::ios::binary); // open file
	// if our file doesn't exist, which should already have been handled
	if(!FileStream)
	{
		FileStream.close();
		FileOpen = 0;
		free(path);
		delete[] Caption;
		return FILE_NOT_FOUND;					
	}	
	// if our file does exist, as it should
	else
	{
		FileStream.seekg(0, std::ios::end);
		MaxAddress = FileStream.tellg();
		// here we determine whether or not we'll use paging
		if( ( MaxAddress > 0x800000 ) && AllowPaging) // if the file is more than 8 megs and we allow paging (default)
		{
			IsPagingOn = 1;
			NumPages = ( MaxAddress - ( MaxAddress % 0x800000 ) ) / 0x80000;
			if(MaxAddress % 0x800000)
				NumPages++;
			File = (unsigned char*)GlobalAlloc(GMEM_FIXED, 0x800000);
		}
		else
		{
			IsPagingOn = 0;
			File = (unsigned char*)GlobalAlloc(GMEM_FIXED , MaxAddress);
		}
		UnicodeFile = (wchar_t*)File;
		if(File == NULL)
		{
			FileStream.close();
			free(path);
			delete[] Caption;
			return MEM_ERROR;
		}
		else
		{
			FileStream.seekg(0, std::ios::beg);
      		if(IsPagingOn)
			{
				FileStream.read((LPSTR)File, 0x800000);
				PagesAltered = new int[NumPages];
				for(int a = 0; a < NumPages; a++)
					PagesAltered[a] = 0;
			}
			else
				FileStream.read((LPSTR)File, MaxAddress);
			SetWindowText(hWnd, Caption);
		    FileOpen = 1;  // end of the !RomFile conditional
			if(IsPagingOn) 
				;
			else
			{
				FileStream.close();
			}
			if( ( GetFileByte(0) == 0xff ) && ( GetFileByte(1) == 0xfe ) )
			{
				UnicodeOutput = true;
				CodePageMenu(hWnd, ID_OPTIONS_TEXTCODEPAGE_UNICODE);
			}
		}
	}
	delete[] Caption;

	// here we'll look for a table file with the same name but with the .tbl extentsion
	_splitpath(path, szDrive, szDir, szName, szExt);
	if(strcmp(szExt, ".tbl"))
	{
		_tcscpy(szExt, ".tbl");
		_makepath(szPath, szDrive, szDir, szName, szExt);
		OpenTableFile(hWnd, szPath);
	}
	FileOpenedMenu();
	wsprintf(temp, "Size: %08X", MaxAddress);
	SendMessage (StatusBar, SB_SETTEXT, 2, (LPARAM)temp);
	free(path);
	return FILE_OPENED;
}

int OpenFile(HWND hWnd, char* name, char* title)
{
	char* Caption, szDrive[_MAX_PATH], szDir[_MAX_PATH], 
			szName[_MAX_PATH], szExt[_MAX_PATH], szPath[_MAX_PATH];
	Caption = new char[MAX_PATH];
	char temp[100];
	// the gave us a file name
	_tcscpy(Filename, name);
					
	// let's put the file name in the window title caption
	wsprintf(Caption, "%s %d.%d.%d - [ ", PROGRAM_NAME, PROGRAM_MAJOR, PROGRAM_MINOR, PROGRAM_BUILD);
	strcat(Caption, title);
	strcat(Caption, " ]");
	_tcscpy(Filename, name);
	_tcscpy(TitleName, title);
	strcat(Caption, " - ");
	if(CleanUp)
		strcat(Caption, "ASCII");
	else
		strcat(Caption, CPages[CurrentCodePage].name); 

	FileStream.open(name, std::ios::in | std::ios::binary); // open file
	// if our file doesn't exist, which should already have been handled
	if(!FileStream)
	{
		FileStream.close();
		FileOpen = 0;
		delete[] Caption;
		return FILE_NOT_FOUND;					
	}	
	// if our file does exist, as it should
	else
	{
		FileStream.seekg(0, std::ios::end);
		MaxAddress = FileStream.tellg();
		if( ( MaxAddress > 0x800000 ) && AllowPaging )
		{
			IsPagingOn = 1;
			NumPages = ( MaxAddress - ( MaxAddress % 0x800000 ) ) / 0x80000;
			if(MaxAddress % 0x800000)
				NumPages++;
			File = (unsigned char*)GlobalAlloc(GMEM_FIXED , 0x800000);
		}
		else 
		{
			File = (unsigned char*)GlobalAlloc(GMEM_FIXED , MaxAddress);
			IsPagingOn = 0;
		}
		UnicodeFile = (wchar_t*)File;
		if(File == NULL)
		{
			FileStream.close();
			delete[] Caption;
			return MEM_ERROR;
		}
		else
		{
			FileStream.seekg(0, std::ios::beg);
			if(IsPagingOn)
			{
				FileStream.read((LPSTR)File, 0x800000 );
				PagesAltered = new int[NumPages];
				for(int a = 0; a < NumPages; a++)
					PagesAltered[a] = 0;
			}
			else
      			FileStream.read((LPSTR)File, MaxAddress);
			SetWindowText(hWnd, Caption);
		    FileOpen = 1;  // end of the !RomFile conditional
			if(IsPagingOn)
				;
			else
			{
				FileStream.close();
			}
			if( ( GetFileByte(0) == 0xff ) && ( GetFileByte(1) == 0xfe ) )
			{
				UnicodeOutput = true;
				CodePageMenu(hWnd, ID_OPTIONS_TEXTCODEPAGE_UNICODE);
			}
		}
	}
	SetWindowText(hWnd, Caption);
	delete[] Caption;
	FileOpenedMenu();

	// here we'll look for a table file with the same name but with the .tbl extentsion
	_splitpath(Filename, szDrive, szDir, szName, szExt);
	if(strcmp(szExt, ".tbl"))
	{
		_tcscpy(szExt, ".tbl");
		_makepath(szPath, szDrive, szDir, szName, szExt);
		OpenTableFile(hWnd, szPath);
	}

	wsprintf(temp, "Size: %08X", MaxAddress);
	SendMessage (StatusBar, SB_SETTEXT, 2, (LPARAM)temp);
	//CurrentTable = -1;
	return FILE_OPENED;
}

int OpenTableFile(HWND hWnd)
{
	std::fstream crapola;
	// structure needed for calling windows' open file dialog
	OPENFILENAME ofn;
	LPTSTR path = new char[MAX_PATH]; 
	_tcscpy((char*)path, "");
	char* Caption = new char[MAX_PATH];
	char temp[500], temp2[500];

	// zero out ofn and fill it with thte wanted values
	ZeroMemory(path, _MAX_PATH);
	ZeroMemory(&ofn, sizeof(OPENFILENAME));
	ZeroMemory(temp, MAX_PATH);
	ZeroMemory(temp2, MAX_PATH);
    ofn.lStructSize			= sizeof(OPENFILENAME);
	ofn.hwndOwner 			= hWnd;
	ofn.lpstrFilter			= (LPSTR)TableFilter;
	ofn.nMaxFile 			= MAX_PATH;
	ofn.nMaxFileTitle		= MAX_PATH;
	ofn.lpstrFile			= path;
	ofn.lpstrFileTitle 		= TableTitleName;
	ofn.lpstrTitle	  		= "Open a table file";
	ofn.Flags				= OFN_EXPLORER | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
				
	if(!GetOpenFileName(&ofn))
	{
		// they cancelled, so let's just leave
		delete[] Caption;
		delete[] path;
		return FILE_CANCELLED;
	}
	crapola.open(path, std::ios::in);

	if(!crapola)
	{
		crapola.close();
		//FileOpen = 0;
		delete[] Caption;
		delete[] path;
		return FILE_NOT_FOUND;					
	}
	// if our file does exist
	else {
		Table[TotalTables] = new table;
		ParseTable(crapola, Table[TotalTables], ofn.lpstrFileTitle);
		_tcscpy(Tablefilename[TotalTables - 1], ofn.lpstrFile);
		Table[TotalTables - 1]->setFilename(ofn.lpstrFile);
	}
	wsprintf(temp2, "%s %d.%d.%d - ", PROGRAM_NAME, PROGRAM_MAJOR, PROGRAM_MINOR, PROGRAM_BUILD);
	wsprintf(temp, "[ %s ] - %s", TitleName, Table[TotalTables - 1]->getName());
	strcat(temp2, temp);
	SetWindowText(hWnd, temp2);
	CurrentTable = TotalTables - 1;

	//EnableMenuItem(HexecuteMenu, ID_SPECIAL_EDITTABLE, MF_ENABLED);
	EnableMenuItem(HexecuteMenu, ID_SPECIAL_DUMPTEXT, MF_ENABLED);
	EnableMenuItem(HexecuteMenu, ID_SPECIAL_INSERTTEXT, MF_ENABLED);
	return FILE_OPENED;
}

int OpenTableFile(HWND hWnd, char* name)
{
	std::fstream crapola;
	LPTSTR path = (LPTSTR)malloc(_MAX_PATH); 
	_tcscpy((char*)path, "");
	char* Caption, szDrive[_MAX_PATH], szDir[_MAX_PATH], 
			szName[_MAX_PATH], szExt[_MAX_PATH];
	char temp[500], temp2[500];

	Caption = new char[MAX_PATH];

	_splitpath(name, szDrive, szDir, szName, szExt);
	strcat(szName, szExt);

	crapola.open(name, std::ios::in);
	// if our file doesn't exist, which should already have been handled
	if(!crapola)
	{
		crapola.close();
		free(path);
		delete[] Caption;
		return FILE_NOT_FOUND;					
	}
	// if our file does exist
	else {
		Table[TotalTables] = new table;
		ParseTable(crapola, Table[TotalTables], name);
		_tcscpy(Tablefilename[TotalTables - 1], name);
		Table[TotalTables - 1]->setFilename(name);
	}
	wsprintf(temp2, "%s %d.%d.%d - ", PROGRAM_NAME, PROGRAM_MAJOR, PROGRAM_MINOR, PROGRAM_BUILD);
	wsprintf(temp, "[ %s ] - %s", TitleName, Table[TotalTables - 1]->getName());
	strcat(temp2, temp);
	SetWindowText(hWnd, temp2);
	CurrentTable = TotalTables - 1;

	//EnableMenuItem(HexecuteMenu, ID_SPECIAL_EDITTABLE, MF_ENABLED);
	EnableMenuItem(HexecuteMenu, ID_SPECIAL_DUMPTEXT, MF_ENABLED);
	EnableMenuItem(HexecuteMenu, ID_SPECIAL_INSERTTEXT, MF_ENABLED);
	free(path);
	return FILE_OPENED;
}