/*
 * Hexecute 3.0
 * PreferencesProc.h
 * copyright 2000 Mike Walston
 *
 * This is the file that holds WinProc for the preferences dialog
 */


#include <windows.h>
#include <commctrl.h>
#include <commdlg.h>
#include <stdio.h>
#include <tchar.h>
#include <assert.h>
#include "resource.h"
#include "hexecute_defines.h"
#include "hexecute_globals.h"
#include "hexecute_types.h"

int CreatePropertySheet(HWND hWndOwner)
{
	PROPSHEETPAGE psp[3];
	PROPSHEETHEADER psh;

	psp[0].dwSize = sizeof(PROPSHEETPAGE);
	psp[0].dwFlags = PSP_USETITLE;
	psp[0].hInstance = hInst;
	psp[0].pszTemplate = MAKEINTRESOURCE(IDP_OPTIONS_DISPLAY);
	psp[0].pszIcon = NULL;
	psp[0].pfnDlgProc = DisplayProc;
	psp[0].pszTitle = "Display";
	psp[0].lParam = 0;

	psp[1].dwSize = sizeof(PROPSHEETPAGE);
	psp[1].dwFlags = PSP_USETITLE;
	psp[1].hInstance = hInst;
	psp[1].pszTemplate = MAKEINTRESOURCE(IDP_OPTIONS_FONTS);
	psp[1].pszIcon = NULL;
	psp[1].pfnDlgProc = FontProc;
	psp[1].pszTitle = "Font";
	psp[1].lParam = 0;

	psp[2].dwSize = sizeof(PROPSHEETPAGE);
	psp[2].dwFlags = PSP_USETITLE;
	psp[2].hInstance = hInst;
	psp[2].pszTemplate = MAKEINTRESOURCE(IDP_OPTIONS_SUBTABLES);
	psp[2].pszIcon = NULL;
	psp[2].pfnDlgProc = SubtableProc;
	psp[2].pszTitle = "General";
	psp[2].lParam = 0;

	psh.dwSize = sizeof(PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW;
	psh.hwndParent = hWndOwner;
	psh.hInstance = hInst;
	psh.pszIcon = NULL;
	psh.pszCaption = "Hexecute Preferences";
	psh.nPages = sizeof (psp) / sizeof(PROPSHEETPAGE);
	psh.ppsp = (LPCPROPSHEETPAGE) &psp;

	return PropertySheet(&psh);
}

BOOL CALLBACK DisplayProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static PROPSHEETPAGE *ps;
//	RECT RectDraw;
	LPDRAWITEMSTRUCT pdis;
	static int AddressColorTemp, AddressBGColorTemp,
		OddColumnColorTemp, EvenColumnColorTemp, HexBGColorTemp,
		TextColorTemp, TextBGColorTemp, HighlightColorTemp, HighlightTextTemp;
	static CHOOSECOLOR cc;
	static COLORREF  crCustColors[16];
	char temp[50];
	//int nihon, wasabi;

	switch(message) {
	case WM_INITDIALOG:
		ps = (PROPSHEETPAGE*) lParam;
		AddressColorTemp = AddressColor; 
		AddressBGColorTemp = AddressBGColor;
		OddColumnColorTemp = OddColumnColor;
		EvenColumnColorTemp = EvenColumnColor;
		HexBGColorTemp = HexBGColor;
		TextColorTemp  = TextColor;
		TextBGColorTemp = TextBGColor;
		HighlightColorTemp = HighlightColor;
		HighlightTextTemp = HighlightTextColor;
		
		SendMessage(GetDlgItem(hWnd, IDC_HEX_GROUPING), CB_ADDSTRING, NULL, (LPARAM)TEXT("Byte"));
		SendMessage(GetDlgItem(hWnd, IDC_HEX_GROUPING), CB_ADDSTRING, NULL, (LPARAM)TEXT("Short Word (2 Bytes)"));
		SendMessage(GetDlgItem(hWnd, IDC_HEX_GROUPING), CB_ADDSTRING, NULL, (LPARAM)TEXT("Double Word (4 Bytes)"));
		SendMessage(GetDlgItem(hWnd, IDC_HEX_GROUPING), CB_ADDSTRING, NULL, (LPARAM)TEXT("Quad Word (8 Bytes)"));

		switch(GroupSize) {
			case 1:
				SendMessage(GetDlgItem(hWnd, IDC_HEX_GROUPING), CB_SELECTSTRING, NULL, (LPARAM)TEXT("Byte"));
				break;
			case 2:
				SendMessage(GetDlgItem(hWnd, IDC_HEX_GROUPING), CB_SELECTSTRING, NULL, (LPARAM)TEXT("Short Word (2 Bytes)"));
				break;
			case 4:
				SendMessage(GetDlgItem(hWnd, IDC_HEX_GROUPING), CB_SELECTSTRING, NULL, (LPARAM)TEXT("Double Word (4 Bytes)"));
				break;
			case 8:
				SendMessage(GetDlgItem(hWnd, IDC_HEX_GROUPING), CB_SELECTSTRING, NULL, (LPARAM)TEXT("Quad Word (8 Bytes)"));
				break;
		}

		cc.lStructSize = sizeof(CHOOSECOLOR);
		cc.hwndOwner = hWnd;
		cc.hInstance = NULL;
		cc.rgbResult = RGB(0x80,0x80,0x80);
		cc.lpCustColors = crCustColors;
		cc.Flags = CC_RGBINIT | CC_FULLOPEN;
		cc.lCustData = 0;
		cc.lpfnHook = NULL;
		cc.lpTemplateName = NULL;
		return true;

	case WM_NOTIFY:
		switch (((NMHDR FAR *)lParam)->code) {

			case PSN_APPLY:
				AddressColor = AddressColorTemp; 
				AddressBGColor = AddressBGColorTemp;
				OddColumnColor = OddColumnColorTemp;
				EvenColumnColor = EvenColumnColorTemp;
				HexBGColor = HexBGColorTemp;
				TextColor  = TextColorTemp;
				TextBGColor = TextBGColorTemp;
				HighlightColor = HighlightColorTemp;
				GetDlgItemText(hWnd, IDC_HEX_GROUPING, temp, 49);
				if(!strcmp(temp, TEXT("Byte")))
					GroupSize = 1;
				else if(!strcmp(temp, TEXT("Short Word (2 Bytes)")))
					GroupSize = 2;
				else if(!strcmp(temp, TEXT("24-bit Word (3 Bytes)")))
					GroupSize = 3;
				else if(!strcmp(temp, TEXT("Double Word (4 Bytes)")))
					GroupSize = 4;
				else if(!strcmp(temp, TEXT("Quad Word (8 Bytes)")))
					GroupSize = 8;
				ImageList_SetBkColor(hImageList, TextBGColor);
				break;
		}

		return true;

	case WM_DRAWITEM:
		pdis = (LPDRAWITEMSTRUCT) lParam;
		switch(pdis->CtlID) {
		case IDC_COLOR_OFFSET:
			FillRect(pdis->hDC, &pdis->rcItem, CreateSolidBrush(AddressColorTemp));
			FrameRect(pdis->hDC, &pdis->rcItem, (HBRUSH)GetStockObject (BLACK_BRUSH));
			break;

		case IDC_COLOR_OFFSETBG:
			FillRect(pdis->hDC, &pdis->rcItem, CreateSolidBrush(AddressBGColorTemp));
			FrameRect(pdis->hDC, &pdis->rcItem, (HBRUSH)GetStockObject (BLACK_BRUSH));
			break;

		case IDC_COLOR_ODDCOLUMN:
			FillRect(pdis->hDC, &pdis->rcItem, CreateSolidBrush(OddColumnColorTemp));
			FrameRect(pdis->hDC, &pdis->rcItem, (HBRUSH)GetStockObject (BLACK_BRUSH));
			break;

		case IDC_COLOR_EVENCOLUMN:
			FillRect(pdis->hDC, &pdis->rcItem, CreateSolidBrush(EvenColumnColorTemp));
			FrameRect(pdis->hDC, &pdis->rcItem, (HBRUSH)GetStockObject (BLACK_BRUSH));
			break;

		case IDC_COLOR_HEXBG:
			FillRect(pdis->hDC, &pdis->rcItem, CreateSolidBrush(HexBGColorTemp));
			FrameRect(pdis->hDC, &pdis->rcItem, (HBRUSH)GetStockObject (BLACK_BRUSH));
			break;

		case IDC_COLOR_TEXT:
			FillRect(pdis->hDC, &pdis->rcItem, CreateSolidBrush(TextColorTemp));
			FrameRect(pdis->hDC, &pdis->rcItem, (HBRUSH)GetStockObject (BLACK_BRUSH));
			break;

		case IDC_COLOR_TEXTBG:
			FillRect(pdis->hDC, &pdis->rcItem, CreateSolidBrush(TextBGColorTemp));
			FrameRect(pdis->hDC, &pdis->rcItem, (HBRUSH)GetStockObject (BLACK_BRUSH));
			break;

		case IDC_COLOR_HIGHLIGHTBG:
			FillRect(pdis->hDC, &pdis->rcItem, CreateSolidBrush(HighlightColorTemp));
			FrameRect(pdis->hDC, &pdis->rcItem, (HBRUSH)GetStockObject (BLACK_BRUSH));
			break;

		case IDC_COLOR_HIGHLIGHT_TEXT:
			FillRect(pdis->hDC, &pdis->rcItem, CreateSolidBrush(HighlightTextTemp));
			FrameRect(pdis->hDC, &pdis->rcItem, (HBRUSH)GetStockObject (BLACK_BRUSH));
			break;
		}
		return true;

	case WM_COMMAND:
		switch(wParam) {
			case IDC_COLOR_OFFSET:
				if(ChooseColor(&cc)) 
					AddressColorTemp = cc.rgbResult;
				break;
			case IDC_COLOR_OFFSETBG:
				if(ChooseColor(&cc)) 
					AddressBGColorTemp = cc.rgbResult;
				break;
			case IDC_COLOR_ODDCOLUMN:
				if(ChooseColor(&cc)) 
					OddColumnColorTemp = cc.rgbResult;
				break;
			case IDC_COLOR_EVENCOLUMN:
				if(ChooseColor(&cc)) 
					EvenColumnColorTemp = cc.rgbResult;
				break;
			case IDC_COLOR_HEXBG:
				if(ChooseColor(&cc)) 
					HexBGColorTemp = cc.rgbResult;
				break;
			case IDC_COLOR_TEXT:
				if(ChooseColor(&cc)) 
					TextColorTemp = cc.rgbResult;
				break;
			case IDC_COLOR_TEXTBG:
				if(ChooseColor(&cc)) 
					TextBGColorTemp = cc.rgbResult;
				break;
			case IDC_COLOR_HIGHLIGHTBG:
				if(ChooseColor(&cc)) 
					HighlightColorTemp = cc.rgbResult;
				break;
			case IDC_LOAD_DEFAULTS:
				AddressColorTemp = 0x00000000; 
				AddressBGColorTemp = 0x00CCCCCC;
				OddColumnColorTemp = 0x007F0000;
				EvenColumnColorTemp = 0x00FF0000;
				HexBGColorTemp = 0x00FFFFFF;
				TextColorTemp  = 0x00000000;
				TextBGColorTemp = 0x00FFFFFF;
				HighlightColorTemp = 0x00BBBBBB;
				InvalidateRect(hWnd, NULL, FALSE);
				break;
		}
	}
	return false;
}

BOOL CALLBACK FontProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	/*
	 *  This keeps crashing in the MIDDLE of ChooseFont(&cf) and
	 *  I'm not sure why, I've tried zeroing out all the structs
	 *  and I just can't seem to figure out why it's doing that
	 *  if it doesn't crash, then the font switching works out okay
	 *  Something in here is ALSO crashing the color selection part
	 *  of the property sheet, this isn't making much sense at all
	 */
	/*
	 *  After a little bit of research, it appears that the problem
	 *  is in CodePagesProc, of all places, how very odd
	 */
	LPDRAWITEMSTRUCT pdis;
	static PROPSHEETPAGE *ps;
	static CHOOSEFONT cf;
	static CHOOSEFONTW cfw;
	static LOGFONTW lfw;
//	HWND hWndTemp;
//	HDC hdcTemp; 
	int f00h3h = 5, tempwhatever, OompaLoompa, o, e;
	TEXTMETRIC TempTM;
	SIZE RectTemp;
	TCHAR codePageText[100];
	static HFONT hFontHexTemp = hFontHex, hFontTemp = hFont, hFontJapaneseTemp = hFontJapanese,
		hFontCodePages[7], hFontUnicodeTemp = hFontUnicode;
	static char CodePageFontNames[7][50], HexFontNameTemp[50], UnicodeFontNameTemp[50];

	static int NumItems = -1;
	static long FontHeightTemp;
	int Selected, a, japanese;

	switch(message) {
	case WM_INITDIALOG:
		// here we make sure we keep the code page fonts and not mess them up
		for(o = 0; o < InstalledPage; o++)
		{
			_tcscpy(CodePageFontNames[o], CPages[o].FontName);
			hFontCodePages[o] = CPages[o].font;
		}
		FontHeightTemp = FontHeight;
		_tcscpy(HexFontNameTemp, HexFontName);
		_tcscpy(UnicodeFontNameTemp, UnicodeFontName);
		hWndFont = hWnd;
		ps = (PROPSHEETPAGE*) lParam;
		SendMessage(GetDlgItem(hWndFont, IDC_CPLIST), LB_ADDSTRING, 0, (LPARAM)TEXT("1252 (Latin I - ANSI)"));
		for(a = 0; a < InstalledPage; a++)
			SendMessage(GetDlgItem(hWndFont, IDC_CPLIST), LB_ADDSTRING, 0, (LPARAM)CPages[a].name);
		SendMessage(GetDlgItem(hWndFont, IDC_CPLIST), LB_ADDSTRING, 0, (LPARAM)TEXT("Unicode"));
		return true;

	case WM_DRAWITEM:
		pdis = (LPDRAWITEMSTRUCT) lParam;
		switch(pdis->CtlID) {
		case IDC_CPSAMPLEBUTTON:
			FillRect(pdis->hDC, &pdis->rcItem, CreateSolidBrush(TextBGColor));
			SelectObject(pdis->hDC, hFontHexTemp);
			GetTextMetrics(pdis->hDC, &TempTM);
			tempwhatever = TempTM.tmHeight + TempTM.tmExternalLeading;
			SetBkColor(pdis->hDC, TextBGColor);
			for(int heeya = 0; heeya < 6 ; heeya++)
			{
				for( int a = 0; a < 4; a++)
				{
					SetTextColor(pdis->hDC, OddColumnColor);
					TextOut(pdis->hDC, f00h3h,5 + (heeya * tempwhatever), TEXT("0000 "), 5);
					GetTextExtentPoint32(pdis->hDC, TEXT("0000 "), strlen(TEXT("0000 ")), &RectTemp);
					f00h3h += RectTemp.cx;
					SetTextColor(pdis->hDC, EvenColumnColor);
					TextOut(pdis->hDC, f00h3h,5 + (heeya * tempwhatever), TEXT("0000 "), 5);
					GetTextExtentPoint32(pdis->hDC, TEXT("0000 "), strlen(TEXT("0000 ")), &RectTemp);
					f00h3h += RectTemp.cx;
				}
				f00h3h = 5;
			}
			FrameRect(pdis->hDC, &pdis->rcItem, (HBRUSH)GetStockObject (BLACK_BRUSH));
			break;

		}
		return true;

	case WM_COMMAND:
		switch(LOWORD(wParam)) {
			case IDC_GTFONT:

				ZeroMemory(&Font, sizeof(LOGFONT));
				Font.lfQuality = DRAFT_QUALITY;
				Font.lfCharSet = DEFAULT_CHARSET;
				_tcscpy(Font.lfFaceName, HexFontName);
				Font.lfHeight = FontHeightTemp;		// the height of the desired font is 12
				Font.lfWidth = 0;                          // use the default width
				Font.lfWeight = FW_NORMAL;						 // use normal thickness
				Font.lfPitchAndFamily = FF_MODERN;	// use a fixed pitch font

				ZeroMemory(&cf, sizeof(CHOOSEFONT));
				cf.lStructSize = sizeof(CHOOSEFONT);
				cf.hwndOwner = hWnd;
				cf.hDC = NULL;
				cf.lpLogFont = &Font;
				cf.iPointSize = 0;
				cf.Flags = CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS | CF_FIXEDPITCHONLY;
				cf.rgbColors = 0;
				cf.lCustData = 0;
				cf.lpfnHook = NULL;
				cf.lpTemplateName = NULL;
				cf.hInstance = hInst;
				cf.lpszStyle = NULL;
				cf.nFontType = 0;
				cf.nSizeMin = 0;
				cf.nSizeMax = 0;

				if(ChooseFont(&cf))
				{
					FontHeightTemp = Font.lfHeight;
					_tcscpy(HexFontNameTemp,Font.lfFaceName);
					hFontHexTemp = CreateFontIndirect(&Font);
				}
		
				InvalidateRect(GetDlgItem(hWnd, IDC_CPSAMPLEBUTTON), NULL, TRUE);
				break;
		
			case IDC_CPFONT:

				// this is the character output font
				ZeroMemory(&Font, sizeof(LOGFONT));
				Font.lfQuality = DRAFT_QUALITY;
				// we check the first value in the list box
				Selected = SendMessage(GetDlgItem(hWnd, IDC_CPLIST), LB_GETSEL, 0, 0);
				OompaLoompa = 0;
				if(!Selected)
				{
					Selected = SendMessage(GetDlgItem(hWnd, IDC_CPLIST), LB_GETSEL, InstalledPage + 1, 0);
					if(!Selected)
					{
						for(OompaLoompa = 0; OompaLoompa < InstalledPage; OompaLoompa++)
						{
							if(SendMessage(GetDlgItem(hWnd, IDC_CPLIST), LB_GETSEL, 1 + OompaLoompa, 0))
								break;
						}
					}
					else
					{
						ZeroMemory(&lfw, sizeof(LOGFONTW));
						lfw.lfQuality = DRAFT_QUALITY;
						lfw.lfCharSet = UNICODE_CHARSET;
						MultiByteToWideChar(1252, MB_PRECOMPOSED, UnicodeFontName, -1, lfw.lfFaceName, 32);
						//swprintf(lfw.lfFaceName, L"%s", UnicodeFontName);
						//MessageBoxW(hWnd, lfw.lfFaceName, LPROGRAM_NAME, MB_OK);
						japanese = 0;
						lfw.lfHeight = FontHeightTemp;		// the height of the desired font is 12
						lfw.lfWidth = 0;                          // use the default width
						lfw.lfWeight = FW_NORMAL;						 // use normal thickness
						lfw.lfPitchAndFamily = FF_MODERN;	// use a fixed pitch font

						ZeroMemory(&cfw, sizeof(CHOOSEFONTW));
						cfw.lStructSize = sizeof(CHOOSEFONTW);
						cfw.hwndOwner = hWnd;
						cfw.hDC = NULL;
						cfw.lpLogFont = &lfw;
						cfw.iPointSize = 0;
						cfw.Flags = CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS;
						cfw.rgbColors = 0;
						cfw.lCustData = 0;
						cfw.lpfnHook = NULL;
						cfw.lpTemplateName = NULL;
						cfw.hInstance = hInst;
						cfw.lpszStyle = NULL;
						cfw.nFontType = 0;
						cfw.nSizeMin = 0;
						cfw.nSizeMax = 0;

						if(ChooseFontW(&cfw))
						{
							Font.lfHeight = FontHeightTemp;
							wsprintf(UnicodeFontNameTemp , "%S", lfw.lfFaceName);
							hFontUnicodeTemp = CreateFontIndirectW(&lfw);
						}
						break;
					}
					OompaLoompa++;
				}
				if(!OompaLoompa)
				{
					//MessageBox(hWnd, TEXT("1252 Selected"), PROGRAM_NAME, MB_OK);
					Font.lfCharSet = DEFAULT_CHARSET;
					_tcscpy(Font.lfFaceName, FontName);
					japanese = 0;
					Font.lfHeight = FontHeightTemp;		// the height of the desired font is 12
					Font.lfWidth = 0;                          // use the default width
					Font.lfWeight = FW_NORMAL;						 // use normal thickness
					Font.lfPitchAndFamily = FF_MODERN;	// use a fixed pitch font
				}
				else
				{
					switch(CPages[OompaLoompa-1].CodePageNum) {
						case 932: // japanese
							Font.lfCharSet = SHIFTJIS_CHARSET;
							_tcscpy(Font.lfFaceName, CPages[OompaLoompa-1].FontName);
							Font.lfHeight = FontHeightTemp;								 // the height of the desired font is 12
							break;
						case 874: // thai
							Font.lfCharSet = THAI_CHARSET;
							_tcscpy(Font.lfFaceName, CPages[OompaLoompa-1].FontName);
							Font.lfHeight = FontHeightTemp;
							break;
						case 936: // simplified chinese
							Font.lfCharSet = CHINESEBIG5_CHARSET;
							_tcscpy(Font.lfFaceName, CPages[OompaLoompa-1].FontName);
							Font.lfHeight = FontHeightTemp;
							break;
						case 949: // Korean
							Font.lfCharSet = HANGUL_CHARSET ;
							_tcscpy(Font.lfFaceName, CPages[OompaLoompa-1].FontName);
							Font.lfHeight = FontHeightTemp;
							break;
						case 950: // Traditional Chinese
							Font.lfCharSet = CHINESEBIG5_CHARSET;
							_tcscpy(Font.lfFaceName, CPages[OompaLoompa-1].FontName);
							Font.lfHeight = FontHeightTemp;
							break;
						/*
						 * not doing johab until all the others are working fine
						 *
						 *

						case 1361: // Korean - Johab
							Font.lfCharSet = JOHAB_CHARSET;
							_tcscpy(Font.lfFaceName, CPages[OompaLoompa-1].FontName);
							Font.lfHeight = FontHeight;
							break;
						 */
					}
				}

				ZeroMemory(&cf, sizeof(CHOOSEFONT));
				cf.lStructSize = sizeof(CHOOSEFONT);
				cf.hwndOwner = hWnd;
				cf.hDC = NULL;
				cf.lpLogFont = &Font;
				cf.iPointSize = 0;
				cf.Flags = CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS;
				cf.rgbColors = 0;
				cf.lCustData = 0;
				cf.lpfnHook = NULL;
				cf.lpTemplateName = NULL;
				cf.hInstance = hInst;
				cf.lpszStyle = NULL;
				cf.nFontType = 0;
				cf.nSizeMin = 0;
				cf.nSizeMax = 0;
				
				if(ChooseFont(&cf))
				{
					if(OompaLoompa) {
						Font.lfHeight = FontHeightTemp;
						_tcscpy(CodePageFontNames[OompaLoompa-1],Font.lfFaceName);
						hFontCodePages[OompaLoompa-1] = CreateFontIndirect(&Font);
					}
					else {
						Font.lfHeight = FontHeightTemp;
						_tcscpy(FontName,Font.lfFaceName);
						hFontTemp = CreateFontIndirect(&Font);
					}
				}
		
				break;


		}
		return true;

	case WM_NOTIFY:
		switch (((NMHDR FAR *)lParam)->code) {

			case PSN_APPLY:
				for(a = 0; a < NumItems; a++)
				{
					Selected = SendMessage(GetDlgItem(hWnd, IDC_CPLIST), LB_GETSEL, a, 0);
					if(Selected)
						break;
				}
				if(a != NumItems && NumItems != -1) {
					SendMessage(GetDlgItem(hWnd, IDC_CPLIST), LB_GETTEXT, a, (LPARAM)&codePageText);
				}
				DeleteObject(hFont);
				DeleteObject(hFontHex);
				DeleteObject(hFontUnicode);
				DeleteObject(hFontJapanese);
				hFontHex = hFontHexTemp;
				hFontUnicode = hFontUnicodeTemp;
				_tcscpy(UnicodeFontName, UnicodeFontNameTemp);
				_tcscpy(HexFontName, HexFontNameTemp);
				hFont = hFontTemp;
				for(e = 0; e < InstalledPage; e++)
				{
					DeleteObject(CPages[e].font);
					CPages[e].font = hFontCodePages[e];
					_tcscpy(CPages[e].FontName, CodePageFontNames[e]);
				}
				hFontJapanese = hFontJapaneseTemp;
				FontHeight = FontHeightTemp;
				RemakeFonts((const int)&FontHeight);
				SendMessage(hWndData, WM_SIZE, NULL, NULL);
				break;

			case PSN_SETACTIVE:
				break;

		}

		return true;
	}
	return false;
}

BOOL CALLBACK SubtableProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message) {
	case WM_INITDIALOG:
		if(ShowSplash)
			CheckDlgButton(hWnd, IDC_SHOWSPLASH_CHECK, BST_CHECKED);
		else CheckDlgButton(hWnd, IDC_SHOWSPLASH_CHECK, BST_UNCHECKED);
		//if(AllowPaging)
		//	CheckDlgButton(hWnd, IDC_PAGECHECK, BST_CHECKED);
		//else 
			CheckDlgButton(hWnd, IDC_PAGECHECK, BST_UNCHECKED);

	case WM_NOTIFY:
		switch (((NMHDR FAR *)lParam)->code) {

			case PSN_APPLY:
				if(IsDlgButtonChecked(hWnd, IDC_SHOWSPLASH_CHECK) == BST_CHECKED)
					ShowSplash = 1;
				else if (IsDlgButtonChecked(hWnd, IDC_SHOWSPLASH_CHECK) == BST_UNCHECKED)
					ShowSplash = 0;
				if(IsDlgButtonChecked(hWnd, IDC_PAGECHECK) == BST_CHECKED)
					AllowPaging = 1;
				else if (IsDlgButtonChecked(hWnd, IDC_PAGECHECK) == BST_UNCHECKED)
					AllowPaging = 0;
				break;

			case PSN_SETACTIVE:
				break;

		}
		return true;
	}
	return false;
}