/*
 * Hexecute 3.0
 * RemakeFonts.cpp
 * copyright 2000 Mike Walston
 *
 * This function changes all the font sizes for all the fonts 
 * so as to keep their sizes the same and not have really nasty
 * looking output due to messed up calculations on dimensions
 */



#include <windows.h>
#include <commctrl.h>
#include <commdlg.h>
#include <stdio.h>
#include <tchar.h>
#include <assert.h>
#include "resource.h"
#include "hexecute_defines.h"
#include "hexecute_globals.h"
#include "hexecute_types.h"

int RemakeFonts(const int &Size) 
{
	int a;

	// this is the character output font
	ZeroMemory(&Font, sizeof(LOGFONT));
	Font.lfQuality = DRAFT_QUALITY;
	Font.lfCharSet = DEFAULT_CHARSET;
	_tcscpy(Font.lfFaceName, FontName); // sets the font face to Courier
	Font.lfHeight = FontHeight;		// the height of the desired font is 12
	Font.lfWidth = 0;                          // use the default width
	Font.lfWeight = FW_NORMAL;						 // use normal thickness
	Font.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;	// use a fixed pitch font
	DeleteObject(hFont);
	hFont = CreateFontIndirect(&Font);	// create the logical font

	// this is the Unicode font
	ZeroMemory(&UnicodeFont, sizeof(LOGFONTW));
	UnicodeFont.lfQuality = DRAFT_QUALITY;
	UnicodeFont.lfCharSet = UNICODE_CHARSET;
	MultiByteToWideChar(1252, MB_PRECOMPOSED, UnicodeFontName, -1, UnicodeFont.lfFaceName, 32);
	//swprintf(UnicodeFont.lfFaceName, L"%s", UnicodeFontName);
	//_tcscpyW (UnicodeFont.lfFaceName, UnicodeFontName);
	UnicodeFont.lfHeight = FontHeight;		// the height of the desired font is 12
	UnicodeFont.lfWidth = 0;                          // use the default width
	UnicodeFont.lfWeight = FW_NORMAL;						 // use normal thickness
	DeleteObject(hFontUnicode);
	hFontUnicode = CreateFontIndirectW(&UnicodeFont);	// create the logical font

	// creating the font for the output of hte hex
	ZeroMemory(&Font, sizeof(LOGFONT));
    Font.lfQuality = DRAFT_QUALITY;
	Font.lfCharSet = DEFAULT_CHARSET;
	_tcscpy(Font.lfFaceName, HexFontName); // sets the font face to Courier
	Font.lfHeight = FontHeight;		// the height of the desired font is 12
	Font.lfWidth = 0;                          // use the default width
	Font.lfWeight = FW_NORMAL;						 // use normal thickness
	Font.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;	// use a fixed pitch font
	DeleteObject(hFontHex);
	hFontHex = CreateFontIndirect(&Font);	// create the logical font

	for( a = 0; a < TotalTables; a++)
	{
		ZeroMemory(&Font, sizeof(LOGFONT));
		Font.lfQuality = DRAFT_QUALITY;
		if(Table[a]->getCodepage() == 932) // japanese
		{
			Font.lfCharSet = SHIFTJIS_CHARSET;
			_tcscpy(Font.lfFaceName, JapaneseFontName);
			Font.lfHeight = FontHeight;								 // the height of the desired font is 12
		}
		else if(Table[a]->getCodepage() == 874) // thai
		{
			Font.lfCharSet = THAI_CHARSET;
			_tcscpy(Font.lfFaceName, ThaiFontName);
			Font.lfHeight = FontHeight;
		}
		else if(Table[a]->getCodepage() == 936) // simplified chinese
		{
			Font.lfCharSet = CHINESEBIG5_CHARSET;
			_tcscpy(Font.lfFaceName, SimplifiedChineseFontName);
			Font.lfHeight = FontHeight;
		}
		else if(Table[a]->getCodepage() == 949) // Korean
		{
			Font.lfCharSet = HANGUL_CHARSET ;
			_tcscpy(Font.lfFaceName, KoreanFontName);
			Font.lfHeight = FontHeight;
		}
		else if(Table[a]->getCodepage() == 950) // Traditional Chinese
		{
			Font.lfCharSet = CHINESEBIG5_CHARSET;
			_tcscpy(Font.lfFaceName, TraditionalChineseFontName);
			Font.lfHeight = FontHeight;
		}

		/*
		 * the johab is like extra credit, not to be done until all the
		 * other stuff is already done
		 *

		else if(Table[a]->getCodepage() == 1361) // Korean - Johab
		{
			Font.lfCharSet = JOHAB_CHARSET;
			_tcscpy(Font.lfFaceName, KoreanFontName);
			Font.lfHeight = FontHeight;
		}
		 */
		else
		{
			Font.lfCharSet = DEFAULT_CHARSET;
			Font.lfHeight = FontHeight;
			_tcscpy(Font.lfFaceName, FontName); // sets the font face to Courierheh
		}
		Font.lfWidth = 0;                          // use the default width
		Font.lfWeight = FW_NORMAL;						 // use normal thickness
		Font.lfPitchAndFamily = FF_MODERN;	// use a fixed pitch font
		DeleteObject(tblFont[a]);
		tblFont[a] = CreateFontIndirect(&Font);	// create the logical font
	}

	for( a = 0; a < InstalledPage; a++)
	{
		ZeroMemory(&Font, sizeof(LOGFONT));
		Font.lfQuality = DRAFT_QUALITY;
		if(CPages[a].CodePageNum == 932) // japanese
		{
			Font.lfCharSet = SHIFTJIS_CHARSET;
			_tcscpy(Font.lfFaceName, JapaneseFontName);
			Font.lfHeight = FontHeight;								 // the height of the desired font is 12
		}
		else if(CPages[a].CodePageNum == 874) // thai
		{
			Font.lfCharSet = THAI_CHARSET;
			_tcscpy(Font.lfFaceName, ThaiFontName);
			Font.lfHeight = FontHeight;
		}
		else if(CPages[a].CodePageNum == 936) // simplified chinese
		{
			Font.lfCharSet = CHINESEBIG5_CHARSET;
			_tcscpy(Font.lfFaceName, SimplifiedChineseFontName);
			Font.lfHeight = FontHeight;
		}
		else if(CPages[a].CodePageNum == 949) // Korean
		{
			Font.lfCharSet = HANGUL_CHARSET ;
			_tcscpy(Font.lfFaceName, KoreanFontName);
			Font.lfHeight = FontHeight;
		}
		else if(CPages[a].CodePageNum == 950) // Traditional Chinese
		{
			Font.lfCharSet = CHINESEBIG5_CHARSET;
			_tcscpy(Font.lfFaceName, TraditionalChineseFontName);
			Font.lfHeight = FontHeight;
		}

		/*
		 * the johab is like extra credit, not to be done until all the
		 * other stuff is already done
		 *

		else if(Table[a]->getCodepage() == 1361) // Korean - Johab
		{
			Font.lfCharSet = JOHAB_CHARSET;
			_tcscpy(Font.lfFaceName, KoreanFontName);
			Font.lfHeight = FontHeight;
		}
		 */
		else
		{
			Font.lfCharSet = DEFAULT_CHARSET;
			Font.lfHeight = FontHeight;
			_tcscpy(Font.lfFaceName, FontName); // sets the font face to Courierheh
		}
		Font.lfWidth = 0;                          // use the default width
		Font.lfWeight = FW_NORMAL;						 // use normal thickness
		Font.lfPitchAndFamily = FF_MODERN;	// use a fixed pitch font
		DeleteObject(CPages[a].font);
		CPages[a].font = CreateFontIndirect(&Font);	// create the logical font
	}

	SendMessage(hWndData, WM_SIZE, NULL, NULL);

	return 0;
}