// Author: Nicolas Livaditis (AKA NightWolve)
// All Rights Reserved 2012
// Included just for hexecute 3 patch
// Non-commercial use only

// One of the fastest ASM functions to copy one buffer to another
// Use SIZE asm operand if need be for count: e.g. SIZE char_array
#define fmemcpy(dest, src, count)\
	__asm CLD \
	__asm MOV   ESI, src \
	__asm MOV   EDI, dest \
	__asm MOV   ECX, count \
	__asm MOV   AL, CL \
	__asm AND   AL, 3 \
	__asm SHR   ECX, 2 \
	__asm REP   MOVSD \
	__asm MOV   CL, AL \
	__asm REP   MOVSB
// For OP_dest or OP_src, use either MOV or LEA if you have a pointer or an array to one
// e.g. fmemcpy2(LEA, array, MOV, lpSrcPtr, 20)
#define fmemcpy2(OP_dest, dest, OP_src, src, count)\
	__asm CLD \
	__asm OP_src   ESI, src \
	__asm OP_dest  EDI, dest \
	__asm MOV   ECX, count \
	__asm MOV   AL, CL \
	__asm AND   AL, 3 \
	__asm SHR   ECX, 2 \
	__asm REP   MOVSD \
	__asm MOV   CL, AL \
	__asm REP   MOVSB
#define fbytecpy(OP_dest, dest, OP_src, src, btCount)\
	__asm CLD \
	__asm XOR   ECX, ECX \
	__asm OP_src   ESI, src \
	__asm OP_dest  EDI, dest \
	__asm MOV   CL, btCount \
	__asm MOV   AL, CL \
	__asm AND   AL, 3 \
	__asm SHR   CL, 2 \
	__asm REP   MOVSD \
	__asm MOV   CL, AL \
	__asm REP   MOVSB
// One of the fastest ASM functions to zero out memory
// For arrays/structs (value-defined, not via pointer) - Careful, use of wrong one will crash!
#define fmemzero(aBuffer, dwBytes)\
	__asm CLD \
	__asm XOR   EAX, EAX \
	__asm LEA   EDI, aBuffer \
	__asm MOV   ECX, dwBytes \
	__asm MOV   BL, CL \
	__asm AND   BL, 3 \
	__asm SHR   ECX, 2 \
	__asm REP   STOSD \
	__asm MOV   CL, BL \
	__asm REP   STOSB
// For pointers to arrays/structs (variable is a value that's a pointer)
#define fpmemzero(pBuffer, dwBytes)\
	__asm	CLD \
	__asm	XOR   EAX, EAX \
	__asm	MOV   EDI, pBuffer \
	__asm	MOV   ECX, dwBytes \
	__asm	MOV   BL, CL \
	__asm	AND   BL, 3 \
	__asm	SHR   ECX, 2 \
	__asm	REP   STOSD \
	__asm	MOV   CL, BL \
	__asm	REP   STOSB
// An even faster version if your buffer is DWORD aligned. You can set dwBytesBy4 equal
// to say "SIZE structobject.member/4" and the result will be hardcoded by the assembler
#define fmemzeroBy4(aBuffer, dwBytesBy4)\
	__asm CLD \
	__asm XOR   EAX, EAX \
	__asm LEA   EDI, aBuffer \
	__asm MOV   ECX, dwBytesBy4 \
	__asm REP   STOSD
// THE FASTEST strcpy routine EVER written!!!!
#define fstrcpybase(OP1, szDestination, OP2, szSource)\
/* Phase 1. Get source string size */ \
	__asm CLD             /* Left to right processing for ESI/EDI string functions */ \
	__asm XOR   AL, AL    /* AL has the target Null byte to scan for */ \
	__asm OP2   EDI, szSource /* EDI has source string for byte scanning */ \
	__asm MOV   ESI, EDI  /* ESI points to the source string for phase 2 */ \
	__asm XOR   ECX, ECX  /* Zero out ECX counter */ \
	__asm dec   ECX       /* We're ready to search for Null up to 0xFFFFFFFF times... ;) */ \
	__asm REPNZ SCASB     /* Scan for Null Byte (up to ECX times) */ \
	__asm NOT   ECX       /* !ECX now has the string size counting Null */ \
/* Phase 2. Copy source string to destination */ \
	__asm MOV   AL, CL    /* Copy low byte of size over */ \
	__asm AND   AL, 3     /* Compute leftoever bytes to copy over 1-3 (< 4) */ \
	__asm SHR   ECX, 2    /* Compute DWORDs to copy over (divide byte count by 4) */ \
	__asm OP1   EDI, szDestination /* EDI points to the destination */ \
	__asm REP   MOVSD     /* Copy string over DWORD at a time for ECX times */ \
	__asm MOV   CL, AL    /* ECX is zero at this point, only need to move a byte */ \
	__asm REP   MOVSB     /* Copy over any remaining bytes 1-3 (< 4) */

#define fstrcpy(szDestination, szSource)\
	fstrcpybase(LEA, szDestination, LEA, szSource);
#define fpstrcpy(lpszDestination, lpszSource)\
	fstrcpybase(MOV, lpszDestination, MOV, lpszSource);

// I think I can proudly boast that this has to be the fastest
// implementation of a 'strlen' function in pure x86 Assembly!!
// Returns: Number of bytes in 0 terminated string via ECX - null is counted
// e.g. fstrlen(szFirstName);
//      __asm MOV nSize, ECX
#define fstrlen(szBuffer)\
	__asm	CLD \
	__asm	XOR   AL, AL \
	__asm	XOR   ECX, ECX \
	__asm	dec   ECX \
	__asm	LEA   EDI, szBuffer \
	__asm	REPNZ SCASB \
	__asm	NOT   ECX
// For string pointers (variable is a value that's a pointer)
#define fpstrlen(pszBuffer)\
	__asm	CLD \
	__asm	XOR   AL, AL \
	__asm	XOR   ECX, ECX \
	__asm	dec   ECX \
	__asm	MOV   EDI, pszBuffer \
	__asm	REPNZ SCASB \
	__asm	NOT   ECX

#define fmalloc(dwBytes) HeapAlloc(GetProcessHeap(), 0, dwBytes)
#define ffree(lpMem) HeapFree(GetProcessHeap(), 0, lpMem)
// Little risky to hand write a full calloc function in inlined ASM, but I think I
// got it jjjust right! Warn: 'JZ $+19' MUST be updated if code below it is changed!
// Usage :     fcalloc(lpMyMemPtr, 500);
// lpMyMemPtr would now point to a mem block of 500 bytes or NULL on alloc failure.
// Nice thing is as long as the variable is 4 bytes (pointer), no stupid type checking occurs (no need for casts)
// I've done it again! One of the THE fastest memory allocation+zeroing routines!
// Note: For huge memory needs (GB Range), VirtualAlloc is what's needed, NOT this!
#define fcalloc(lpMem, dwBytes)\
	__asm MOV   EAX, dwBytes \
	__asm PUSH  EAX \
	__asm XOR   EAX, EAX \
	__asm PUSH  EAX \
	__asm CALL  DWORD PTR [GetProcessHeap] \
	__asm PUSH  EAX \
	__asm CALL  DWORD PTR [HeapAlloc] \
	__asm MOV   ECX, dwBytes \
	__asm MOV   ESI, EAX /* Mem location is return via ESI */ \
	__asm OR    EAX, EAX /* Test for NULL return - mem alloc failure */ \
	__asm JZ    $+19 /*Code below IS EXACTLY 19 bytes - Don't mess with this JMP!*/ \
	__asm CLD \
	__asm MOV   EDI, EAX \
	__asm XOR   EAX, EAX \
	__asm MOV   BL, CL \
	__asm AND   BL, 3 \
	__asm SHR   ECX, 2 \
	__asm REP   STOSD \
	__asm MOV   CL, BL \
	__asm REP   STOSB \
	__asm MOV   lpMem, ESI /*No need for messy casts! (19 byte JMP lands up right here IF NULL!)*/
