/*
 * Hexecute 3.0
 * hexecute_types.h
 * copyright 2000 Mike Walston
 *
 * all hexecute custom data types are defined in this file
 * all of this should be platform independant
 */

// if we don't have the file included yet, let's include it
#ifndef _HEXECUTE_TYPES
#define _HEXECUTE_TYPES

#include <stdio.h>

/*
 * #define ARROW 0x8005
 * #define FIRST_HALF_HEX 0x8006
 * #define SECOND_HALF_HEX 0x8007
 * #define ASCII_CHAR 0x8008
 * #define TABLE_BASED_DIRECT_INPUT 0x8008
 * #define SCRIPT_INSERTION 0x8009
 * #define TAB 0x800A
 * #define ADDRESS_VIEW_TYPECHANGE 0x800b
 */

typedef unsigned char ubyte;

struct search_struct{
	ubyte byte[0x1000];
	int size;
};

struct ChangeStruct{
	unsigned long Address;
	unsigned char * Data;
};

struct RANGE {
	long start;
	long end;
};

struct Page {
	long offset;
};

class HChanges {
	public:
	HChanges();
	private:
	ChangeStruct** Changes;
};


class HFile {
	public:
	HFile();
	bool open(char*, bool, bool);
	bool close();
	char GetByte(int);
	bool SetByte(int, char);
	wchar_t GetUnicode(int);
	bool SetUnicode(int, wchar_t);
	private:
	unsigned char* Data;
	FILE* File;
	int PageStart;
	int PageEnd;
	int PageSize;
	bool PagingOn;
	HChanges Changes;
};

// data will point to either a cchange pointer, 
// a wcchange poitner, or a script insertion pointer
struct historyevent {
	char* data; 
	int offset;
	int size;
};

struct DumpStruct {
	long offset;
	long length;
};

struct CodePage {
	TCHAR* name;
	UINT CodePageNum;
	HFONT font;
	LOGFONT logFont;
	TCHAR FontName[100];
};

// this will be used for the opcode analysis, which
// means you I can basically implement tracing at a 
// very rudimentary level
struct opcode {
	TCHAR name[20]; // the name of the opcode
	unsigned int bytes; // number of extra bytes the opcode takes, 0,1,2...
};

typedef struct tableEntry {
	unsigned int value;
//	short	width;
	TCHAR* string;
} tableEntry;
// this is the actual table, this is what the program interfaces with
//


class historyStack {
public:
	historyStack();
	historyStack(int);
	~historyStack() { delete [] stackPtr; };
	bool push(const historyevent&);
	bool pop(historyevent&);
private:
	int size;
	int top;
	historyevent *stackPtr;

	bool isEmpty() const { return top == -1; }
	bool isFull() const { return top == size - 1; }
};

class table {
	friend BOOL CALLBACK TableEditorProc(HWND, UINT, WPARAM, LPARAM);
	friend BOOL CALLBACK TableFileEditorEditWindowProc(HWND, UINT, WPARAM, LPARAM);
	friend BOOL CALLBACK NewSubTableProc(HWND, UINT, WPARAM, LPARAM);
	
public:
	table();	// default constructor
	~table();	//destructor
	tableEntry* getNode(int) const;	// returns the node that holds the string
	tableEntry* getNode32(int) const;
	tableEntry* getNode24(int) const;
	tableEntry* getNode16(int) const;
	tableEntry* getNode8(int) const;
	tableEntry* getMatchingNode(char*) const;
	int Append(int, char*, int);
	int AddRange(int, int, int);
	int AddSubTable(int, int);
	bool Allocate(int*, int);
	bool newNode(int, char*);
	int IsSubTableKey(int, int);
	int IsSubTableEndValue(char);
	bool IsStartFlag(int);
	bool IsEndFlag(int);
	bool IsInSubTableRange(int);
	bool setCodepage(int);
	bool setName(char*);
	bool HasBeenChanged();
	int GenerateTable(char*);
	int SetSubTable(int);
	int SetSubTableKey(int, int);
	int SetSubTableEndKey(int, int);
	int SetSubTableEndAddress(int);
	int SetSubTableStartAddress(int);
	int GetAddressSubTable(int);
	int SetFlags(int);
	int ClearFlags(int);
	int FindSubStrings();
	int getWidestValue();
	int setFilename(char*);
	int getLongestEntry();
	int getSubTable();
	int getSubTableKey();
	int getCodepage();
	TCHAR* getName();
	void dumpNodes(char*);
	void sort();
	void sortForInsertion();
private:
	void ValueInsertionSort(int);
	void StringInsertionSort(int);
	void ValueHeapSort(int);
	void StringHeapSort(int);
	void ValueBubbleSort(int);
	void StringBubbleSort(int);
	void Valuedownheap(tableEntry*, int, int);
	void Stringdownheap(tableEntry*, int, int);
	void swap(tableEntry*, int, int);
	bool isFirstAllocated;
	bool changed;
	int	widestValue;
	int SubTables;
	int CurrentSubTable;
	int SubTableStart[10];
	int SubTableEnd[10];
	int SubTableKeys[10];
	int SubTableEndKeys[10];
	char Filename[MAX_PATH];
	unsigned int longest_entry;
	UINT codepage;
	char name[50];
	tableEntry** tableData;
	tableEntry** InsertionTableData;
	RANGE** Ranges;
	int RangeCount[10];
	DWORD dwFlags;
	int Entries[10];
	int Entries32[10];
	int Entries24[10];
	int Entries16[10];
	int Entries8[10];
	int currentEntries[10];
};

#endif _HEXECUTE_TYPES